@@ -25,6 +25,10 @@
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+   You can contact the author at :
+   - LZ4 homepage : http://fastcompression.blogspot.com/p/lz4.html
+   - LZ4 source repository : http://code.google.com/p/lz4/
 */
 
 //**************************************
@@ -65,14 +69,15 @@
 //**************************************
 // 32 or 64 bits ?
 #if (defined(__x86_64__) || defined(__x86_64) || defined(__amd64__) || defined(__amd64) || defined(__ppc64__) || defined(_WIN64) || defined(__LP64__) || defined(_LP64) )   // Detects 64 bits mode
-#define LZ4_ARCH64 1
+#  define LZ4_ARCH64 1
 #else
-#define LZ4_ARCH64 0
+#  define LZ4_ARCH64 0
 #endif
 
-// Little Endian or Big Endian ? 
+// Little Endian or Big Endian ?
+// Note : overwrite the below #define if you know your architecture endianess
 #if (defined(__BIG_ENDIAN__) || defined(__BIG_ENDIAN) || defined(_BIG_ENDIAN) || defined(_ARCH_PPC) || defined(__PPC__) || defined(__PPC) || defined(PPC) || defined(__powerpc__) || defined(__powerpc) || defined(powerpc) || ((defined(__BYTE_ORDER__)&&(__BYTE_ORDER__ == __ORDER_BIG_ENDIAN__))) )
-#define LZ4_BIG_ENDIAN 1
+#  define LZ4_BIG_ENDIAN 1
 #else
 // Little Endian assumed. PDP Endian and other very rare endian format are unsupported.
 #endif
@@ -81,12 +86,13 @@
 // For others CPU, the compiler will be more cautious, and insert extra code to ensure aligned access is respected
 // If you know your target CPU supports unaligned memory access, you may want to force this option manually to improve performance
 #if defined(__ARM_FEATURE_UNALIGNED)
-#define LZ4_FORCE_UNALIGNED_ACCESS 1
+#  define LZ4_FORCE_UNALIGNED_ACCESS 1
 #endif
 
-// Uncomment this parameter if your target system or compiler does not support hardware bit count
-//#define LZ4_FORCE_SW_BITCOUNT
-
+// Define this parameter if your target system or compiler does not support hardware bit count
+#if defined(_MSC_VER) && defined(_WIN32_WCE)            // Visual Studio for Windows CE does not support Hardware bit count
+#  define LZ4_FORCE_SW_BITCOUNT
+#endif
 
 
 //**************************************
@@ -95,26 +101,32 @@
 #if __STDC_VERSION__ >= 199901L // C99
 /* "restrict" is a known keyword */
 #else
-#define restrict // Disable restrict
+#  define restrict // Disable restrict
 #endif
 
 #define GCC_VERSION (__GNUC__ * 100 + __GNUC_MINOR__)
 
 #ifdef _MSC_VER  // Visual Studio
-#define inline __forceinline // Visual is not C99, but supports some kind of inline
-#include <intrin.h>          // _BitScanForward
+#  define inline __forceinline           // Visual is not C99, but supports some kind of inline
+#  if LZ4_ARCH64	// 64-bit
+#    pragma intrinsic(_BitScanForward64) // For Visual 2005
+#    pragma intrinsic(_BitScanReverse64) // For Visual 2005
+#  else
+#    pragma intrinsic(_BitScanForward)   // For Visual 2005
+#    pragma intrinsic(_BitScanReverse)   // For Visual 2005
+#  endif
 #endif
 
-#ifdef _MSC_VER 
-#define bswap16(x) _byteswap_ushort(x)
+#ifdef _MSC_VER
+#  define lz4_bswap16(x) _byteswap_ushort(x)
 #else
-#define bswap16(x) ((unsigned short int) ((((x) >> 8) & 0xffu) | (((x) & 0xffu) << 8)))
+#  define lz4_bswap16(x) ((unsigned short int) ((((x) >> 8) & 0xffu) | (((x) & 0xffu) << 8)))
 #endif
 
 #if (GCC_VERSION >= 302) || (__INTEL_COMPILER >= 800) || defined(__clang__)
-# define expect(expr,value)    (__builtin_expect ((expr),(value)) )
+#  define expect(expr,value)    (__builtin_expect ((expr),(value)) )
 #else
-# define expect(expr,value)    (expr)
+#  define expect(expr,value)    (expr)
 #endif
 
 #define likely(expr)     expect((expr) != 0, 1)
@@ -133,22 +145,22 @@
 // Basic Types
 //**************************************
 #if defined(_MSC_VER)    // Visual Studio does not support 'stdint' natively
-#define BYTE	unsigned __int8
-#define U16		unsigned __int16
-#define U32		unsigned __int32
-#define S32		__int32
-#define U64		unsigned __int64
+#  define BYTE	unsigned __int8
+#  define U16		unsigned __int16
+#  define U32		unsigned __int32
+#  define S32		__int32
+#  define U64		unsigned __int64
 #else
-#include <stdint.h>
-#define BYTE	uint8_t
-#define U16		uint16_t
-#define U32		uint32_t
-#define S32		int32_t
-#define U64		uint64_t
+#  include <stdint.h>
+#  define BYTE	uint8_t
+#  define U16		uint16_t
+#  define U32		uint32_t
+#  define S32		int32_t
+#  define U64		uint64_t
 #endif
 
 #ifndef LZ4_FORCE_UNALIGNED_ACCESS
-#pragma pack(push, 1) 
+#  pragma pack(push, 1)
 #endif
 
 typedef struct _U16_S { U16 v; } U16_S;
@@ -156,7 +168,7 @@ typedef struct _U32_S { U32 v; } U32_S;
 typedef struct _U64_S { U64 v; } U64_S;
 
 #ifndef LZ4_FORCE_UNALIGNED_ACCESS
-#pragma pack(pop) 
+#  pragma pack(pop)
 #endif
 
 #define A64(x) (((U64_S *)(x))->v)
@@ -194,31 +206,31 @@ typedef struct _U64_S { U64 v; } U64_S;
 // Architecture-specific macros
 //**************************************
 #if LZ4_ARCH64	// 64-bit
-#define STEPSIZE 8
-#define UARCH U64
-#define AARCH A64
-#define LZ4_COPYSTEP(s,d)		A64(d) = A64(s); d+=8; s+=8;
-#define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d)
-#define LZ4_SECURECOPY(s,d,e)	if (d<e) LZ4_WILDCOPY(s,d,e)
-#define HTYPE U32
-#define INITBASE(base)			const BYTE* const base = ip
+#  define STEPSIZE 8
+#  define UARCH U64
+#  define AARCH A64
+#  define LZ4_COPYSTEP(s,d)		A64(d) = A64(s); d+=8; s+=8;
+#  define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d)
+#  define LZ4_SECURECOPY(s,d,e)	if (d<e) LZ4_WILDCOPY(s,d,e)
+#  define HTYPE U32
+#  define INITBASE(base)			const BYTE* const base = ip
 #else		// 32-bit
-#define STEPSIZE 4
-#define UARCH U32
-#define AARCH A32
-#define LZ4_COPYSTEP(s,d)		A32(d) = A32(s); d+=4; s+=4;
-#define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d); LZ4_COPYSTEP(s,d);
-#define LZ4_SECURECOPY			LZ4_WILDCOPY
-#define HTYPE const BYTE*
-#define INITBASE(base)			const int base = 0
+#  define STEPSIZE 4
+#  define UARCH U32
+#  define AARCH A32
+#  define LZ4_COPYSTEP(s,d)		A32(d) = A32(s); d+=4; s+=4;
+#  define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d); LZ4_COPYSTEP(s,d);
+#  define LZ4_SECURECOPY			LZ4_WILDCOPY
+#  define HTYPE const BYTE*
+#  define INITBASE(base)			const int base = 0
 #endif
 
 #if (defined(LZ4_BIG_ENDIAN) && !defined(BIG_ENDIAN_NATIVE_BUT_INCOMPATIBLE))
-#define LZ4_READ_LITTLEENDIAN_16(d,s,p) { U16 v = A16(p); v = bswap16(v); d = (s) - v; }
-#define LZ4_WRITE_LITTLEENDIAN_16(p,i) { U16 v = (U16)(i); v = bswap16(v); A16(p) = v; p+=2; }
+#  define LZ4_READ_LITTLEENDIAN_16(d,s,p) { U16 v = A16(p); v = lz4_bswap16(v); d = (s) - v; }
+#  define LZ4_WRITE_LITTLEENDIAN_16(p,i)  { U16 v = (U16)(i); v = lz4_bswap16(v); A16(p) = v; p+=2; }
 #else		// Little Endian
-#define LZ4_READ_LITTLEENDIAN_16(d,s,p) { d = (s) - A16(p); }
-#define LZ4_WRITE_LITTLEENDIAN_16(p,v) { A16(p) = v; p+=2; }
+#  define LZ4_READ_LITTLEENDIAN_16(d,s,p) { d = (s) - A16(p); }
+#  define LZ4_WRITE_LITTLEENDIAN_16(p,v)  { A16(p) = v; p+=2; }
 #endif
 
 
@@ -253,7 +265,7 @@ inline static int LZ4_NbCommonBytes (register U64 val)
     _BitScanReverse64( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_clzll(val) >> 3); 
+    return (__builtin_clzll(val) >> 3);
     #else
 	int r;
 	if (!(val>>32)) { r=4; } else { r=0; val>>=32; }
@@ -267,7 +279,7 @@ inline static int LZ4_NbCommonBytes (register U64 val)
     _BitScanForward64( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_ctzll(val) >> 3); 
+    return (__builtin_ctzll(val) >> 3);
     #else
 	static const int DeBruijnBytePos[64] = { 0, 0, 0, 0, 0, 1, 1, 2, 0, 3, 1, 3, 1, 4, 2, 7, 0, 2, 3, 6, 1, 5, 3, 5, 1, 3, 4, 4, 2, 5, 6, 7, 7, 0, 1, 2, 3, 3, 4, 6, 2, 6, 5, 5, 3, 4, 5, 6, 7, 1, 2, 4, 6, 4, 4, 5, 7, 2, 6, 5, 7, 6, 7, 7 };
 	return DeBruijnBytePos[((U64)((val & -val) * 0x0218A392CDABBD3F)) >> 58];
@@ -285,7 +297,7 @@ inline static int LZ4_NbCommonBytes (register U32 val)
     _BitScanReverse( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_clz(val) >> 3); 
+    return (__builtin_clz(val) >> 3);
     #else
 	int r;
 	if (!(val>>16)) { r=2; val>>=8; } else { r=0; val>>=24; }
@@ -298,7 +310,7 @@ inline static int LZ4_NbCommonBytes (register U32 val)
     _BitScanForward( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_ctz(val) >> 3); 
+    return (__builtin_ctz(val) >> 3);
     #else
 	static const int DeBruijnBytePos[32] = { 0, 0, 3, 0, 3, 1, 3, 0, 3, 2, 2, 1, 3, 2, 0, 1, 3, 3, 1, 2, 2, 2, 2, 0, 3, 1, 2, 0, 1, 0, 1, 1 };
 	return DeBruijnBytePos[((U32)((val & -(S32)val) * 0x077CB531U)) >> 27];
@@ -328,7 +340,7 @@ int LZ4_compressCtx(void** ctx,
 				 const char* source,
 				 char* dest,
 				 int isize)
-{	
+{
 #if HEAPMODE
 	struct refTables *srt = (struct refTables *) (*ctx);
 	HTYPE* HashTable;
@@ -336,7 +348,7 @@ int LZ4_compressCtx(void** ctx,
 	HTYPE HashTable[HASHTABLESIZE] = {0};
 #endif
 
-	const BYTE* ip = (BYTE*) source;       
+	const BYTE* ip = (BYTE*) source;
 	INITBASE(base);
 	const BYTE* anchor = ip;
 	const BYTE* const iend = ip + isize;
@@ -344,16 +356,16 @@ int LZ4_compressCtx(void** ctx,
 #define matchlimit (iend - LASTLITERALS)
 
 	BYTE* op = (BYTE*) dest;
-	
+
 	int len, length;
 	const int skipStrength = SKIPSTRENGTH;
 	U32 forwardH;
 
 
-	// Init 
+	// Init
 	if (isize<MINLENGTH) goto _last_literals;
 #if HEAPMODE
-	if (*ctx == NULL) 
+	if (*ctx == NULL)
 	{
 		srt = (struct refTables *) malloc ( sizeof(struct refTables) );
 		*ctx = (void*) srt;
@@ -368,9 +380,9 @@ int LZ4_compressCtx(void** ctx,
 	// First Byte
 	HashTable[LZ4_HASH_VALUE(ip)] = ip - base;
 	ip++; forwardH = LZ4_HASH_VALUE(ip);
-	
+
 	// Main Loop
-    for ( ; ; ) 
+    for ( ; ; )
 	{
 		int findMatchAttempts = (1U << skipStrength) + 3;
 		const BYTE* forwardIp = ip;
@@ -398,7 +410,7 @@ int LZ4_compressCtx(void** ctx,
 		// Encode Literal length
 		length = ip - anchor;
 		token = op++;
-		if (length>=(int)RUN_MASK) { *token=(RUN_MASK<<ML_BITS); len = length-RUN_MASK; for(; len > 254 ; len-=255) *op++ = 255; *op++ = (BYTE)len; } 
+		if (length>=(int)RUN_MASK) { *token=(RUN_MASK<<ML_BITS); len = length-RUN_MASK; for(; len > 254 ; len-=255) *op++ = 255; *op++ = (BYTE)len; }
 		else *token = (length<<ML_BITS);
 
 		// Copy Literals
@@ -426,7 +438,7 @@ _endCount:
 		// Encode MatchLength
 		len = (ip - anchor);
 		if (len>=(int)ML_MASK) { *token+=ML_MASK; len-=ML_MASK; for(; len > 509 ; len-=510) { *op++ = 255; *op++ = 255; } if (len > 254) { len-=255; *op++ = 255; } *op++ = (BYTE)len; }
-		else *token += len;	
+		else *token += len;
 
 		// Test end of chunk
 		if (ip > mflimit) { anchor = ip;  break; }
@@ -440,7 +452,7 @@ _endCount:
 		if ((ref > ip - (MAX_DISTANCE + 1)) && (A32(ref) == A32(ip))) { token = op++; *token=0; goto _next_match; }
 
 		// Prepare next loop
-		anchor = ip++; 
+		anchor = ip++;
 		forwardH = LZ4_HASH_VALUE(ip);
 	}
 
@@ -493,10 +505,10 @@ int LZ4_compress64kCtx(void** ctx,
 	U32 forwardH;
 
 
-	// Init 
+	// Init
 	if (isize<MINLENGTH) goto _last_literals;
 #if HEAPMODE
-	if (*ctx == NULL) 
+	if (*ctx == NULL)
 	{
 		srt = (struct refTables *) malloc ( sizeof(struct refTables) );
 		*ctx = (void*) srt;
@@ -568,7 +580,7 @@ _endCount:
 		// Encode MatchLength
 		len = (ip - anchor);
 		if (len>=(int)ML_MASK) { *token+=ML_MASK; len-=ML_MASK; for(; len > 509 ; len-=510) { *op++ = 255; *op++ = 255; } if (len > 254) { len-=255; *op++ = 255; } *op++ = (BYTE)len; }
-		else *token += len;	
+		else *token += len;
 
 		// Test end of chunk
 		if (ip > mflimit) { anchor = ip;  break; }
@@ -595,7 +607,7 @@ _last_literals:
 		else *op++ = (lastRun<<ML_BITS);
 		memcpy(op, anchor, iend - anchor);
 		op += iend-anchor;
-	} 
+	}
 
 	// End
 	return (int) (((char*)op)-dest);
@@ -637,7 +649,7 @@ int LZ4_compress(const char* source,
 int LZ4_uncompress(const char* source,
 				 char* dest,
 				 int osize)
-{	
+{
 	// Local Variables
 	const BYTE* restrict ip = (const BYTE*) source;
 	const BYTE* restrict ref;
@@ -663,7 +675,7 @@ int LZ4_uncompress(const char* source,
 		cpy = op+length;
 		if unlikely(cpy>oend-COPYLENGTH)
 		{
-			if (cpy > oend) goto _output_error;
+			if (cpy > oend) goto _output_error;          // Error : request to write beyond destination buffer
 			memcpy(op, ip, length);
 			ip += length;
 			break;    // Necessarily EOF
@@ -672,7 +684,7 @@ int LZ4_uncompress(const char* source,
 
 		// get offset
 		LZ4_READ_LITTLEENDIAN_16(ref,cpy,ip); ip+=2;
-		if (ref < (BYTE* const)dest) goto _output_error;		
+		if (ref < (BYTE* const)dest) goto _output_error;   // Error : offset create reference outside destination buffer
 
 		// get matchlength
 		if ((length=(token&ML_MASK)) == ML_MASK) { for (;*ip==255;length+=255) {ip++;} length += *ip++; }
@@ -697,7 +709,7 @@ int LZ4_uncompress(const char* source,
 		cpy = op + length - (STEPSIZE-4);
 		if (cpy>oend-COPYLENGTH)
 		{
-			if (cpy > oend) goto _output_error;	
+			if (cpy > oend) goto _output_error;             // Error : request to write beyond destination buffer
 			LZ4_SECURECOPY(ref, op, (oend-COPYLENGTH));
 			while(op<cpy) *op++=*ref++;
 			op=cpy;
@@ -722,7 +734,7 @@ int LZ4_uncompress_unknownOutputSize(
 				char* dest,
 				int isize,
 				int maxOutputSize)
-{	
+{
 	// Local Variables
 	const BYTE* restrict ip = (const BYTE*) source;
 	const BYTE* const iend = ip + isize;
@@ -749,19 +761,19 @@ int LZ4_uncompress_unknownOutputSize(
 		cpy = op+length;
 		if ((cpy>oend-COPYLENGTH) || (ip+length>iend-COPYLENGTH))
 		{
-			if (cpy > oend) goto _output_error;
-			if (ip+length > iend) goto _output_error;
+			if (cpy > oend) goto _output_error;          // Error : request to write beyond destination buffer
+			if (ip+length > iend) goto _output_error;    // Error : request to read beyond source buffer
 			memcpy(op, ip, length);
 			op += length;
 			ip += length;
-			if (ip<iend) goto _output_error;
+			if (ip<iend) goto _output_error;             // Error : LZ4 format violation
 			break;    // Necessarily EOF, due to parsing restrictions
 		}
 		LZ4_WILDCOPY(ip, op, cpy); ip -= (op-cpy); op = cpy;
 
 		// get offset
 		LZ4_READ_LITTLEENDIAN_16(ref,cpy,ip); ip+=2;
-		if (ref < (BYTE* const)dest) goto _output_error;
+		if (ref < (BYTE* const)dest) goto _output_error;   // Error : offset creates reference outside of destination buffer
 
 		// get matchlength
 		if ((length=(token&ML_MASK)) == ML_MASK) { while (ip<iend) { int s = *ip++; length +=s; if (s==255) continue; break; } }
@@ -786,7 +798,7 @@ int LZ4_uncompress_unknownOutputSize(
 		cpy = op + length - (STEPSIZE-4);
 		if (cpy>oend-COPYLENGTH)
 		{
-			if (cpy > oend) goto _output_error;	
+			if (cpy > oend) goto _output_error;           // Error : request to write outside of destination buffer
 			LZ4_SECURECOPY(ref, op, (oend-COPYLENGTH));
 			while(op<cpy) *op++=*ref++;
 			op=cpy;
